//|----------------------------------------------|
//| Mastered Espers Icon Hack                    |
//| by: madsiur                                  |
//| version: 1.1                                 |
//| Released on: March 27, 2022                  |
//| apply to: FF3us 1.0 (no header)              |
//|----------------------------------------------|

// assemble with bass v14, e.g. "bass -o rom.smc mastered-blue-ran.asm"
// https://www.romhacking.net/utilities/794/

//|----------------------------------------------|
//| variables                                    |
//|----------------------------------------------|

constant STAR_ENABLED(1)        // value of 1 to add star icon, other values disable insertion
constant ICON($7F)              // star icon value (can be changed to another icon)

//|----------------------------------------------|
//| functions                                    |
//|----------------------------------------------|

arch snes.cpu

// set base and origin
macro seek(variable offset) {
    origin (offset & $3FFFFF)
    base offset
}

//|----------------------------------------------|
//| star icon                                    |
//|----------------------------------------------|

if STAR_ENABLED == 1 {
    seek($C487B0)
    insert "star.bin"
}

//|----------------------------------------------|
//| bank $C3 code                                |
//|----------------------------------------------|

// in init skills menu function

seek($C31B5E)
    jsl SET_SPELL_OFFSET        // calculate actor's spell starting RAM offset

// in draw esper's name and MP cost function

seek($C35509)
    jsl SET_POSITION            // save star icon X position
    jsr $809F                   // compute map ptr
    rep #$20                    // 16-bit A

seek($C3552E)
    jsl CHECK_MASTERED          // check if current esper is mastered
    jsr $7FD9                   // draw string
    jsl ADD_ICON                // add blue star icon
    jsr $7FD9                   // draw string
    rts

//|----------------------------------------------|
//| outside bank $C3 code                        |
//|----------------------------------------------|

seek($EEAF01)                   // you can change this free space offset
                                // as long as it's not in bank $C3
                                // and that you have 191 ($BF) bytes of free space

// save star icon X position

scope SET_POSITION: {
    rep #$20                    // 16-bit A
    txa                         // transfer X position in A
    clc
    adc.w #$000C                // add 12 to it
    sta.b $F5                   // save icon position
    sep #$20                    // 8-bit A
    lda $E6                     // BG1 write row
    inc                         // go 1 row down
    rtl
}

// calculate actor's spell starting RAM offset

scope SET_SPELL_OFFSET: {
    tdc                         // clear A
    lda.b $28                   // load slot ID (0-3)
    tax                         // index it
    lda $69,x                   // loac actor ID in slot
    xba
    lda.b #$36                  // 54 spells
    rep #$20                    // 16-bit A
    sta.l $004202               // prepare multiplication (actor ID * 54)
    nop
    nop
    nop
    nop
    lda.l $004216               // load multiplication result
    sta.w $0203                 // save it
    sep #$20                    // 8-bit A
    tdc                         // clear A
    lda.b $28                   // load slot ID (0-3)
    tax                         // index it
    rtl
}

// check if current esper is mastered

scope CHECK_MASTERED: {
    phx
    phy
    tdc                         // clear A
    sta.b $FB                   // clear mastered esper byte
    lda $7E9D89,x               // load esper ID
    rep #$20                    // 16-bit A
    sta.b $FC                   // save it
    asl                         // x2
    sta.b $FE                   // save it
    asl                         // x4
    asl                         // x8
    clc
    adc.b $FE                   // x10
    clc
    adc.b $FC                   // x11
    tax
    stz.b $FC
    ldy.w #$0005                // 5 spells max per esper
    sep #$20                    // 8-bit A
loop:
    tdc                         // clear A
    lda $D86E01,x               // esper spell
    cmp #$FF                    // compare to null entry
    beq no_esper                // exit if no esper
    sta.b $FC                   // save spell ID
    rep #$20                    // 16-bit A
    lda.w $0203                 // load current character spell offset
    clc                         // clear carry
    adc.b $FC                   // spell offset + spell ID
    phx                         // save esper data index
    tax                         // set X as spell learnt percentage
    sep #$20                    // 8-bit A
    lda.w $1A6E,x               // load spell learnt percentage
    plx                         // restore esper data index
    cmp.b #$FF                  // compare learnt rate to 100%
    bne not_mastered            // branch if not 100%
    inx
    inx                         // go to next esper spell
    dey                         // decrement loop
    bne loop
no_esper:
    inc.b $FB                   // set esper as mastered
not_mastered:
    ply
    plx
    lda $7E9D89,x               // load esper ID
    rtl
}

// add blue star icon

scope ADD_ICON: {
    ldx.b $F5                   // icon X position
    lda.b $E6                   // BG1 write row
    inc                         // go 1 row down
    xba
    lda.b $00
    xba                         // clear high byte
    rep #$20                    // 16-bit A
    asl
    asl
    asl
    asl
    asl
    asl                         // x64
    sta.b $E7                   // save result
    txa                         // x position
    asl                         // x2
    clc
    adc.b $E7                   // add Y * 64
    adc.w #$3849                // add base address
    sta.l $7E9E89               // set position
    sep #$20                    // 8-bit A
    lda.b $29                   // load palette
    cmp.b #$20                  // check if user's
    bne is_grey                 // branch if esper name is in grey
    lda.b #$24                  // load blue palette
    sta.b $29                   // save it
is_grey:
    ldx.w #$9E8B                // $7E9E8B
    stx.w $2181                 // set WRAM low bytes
    lda.b $FB                   // load mastered esper byte
    beq not_mastered            // branch if not mastered
    lda.b #ICON                 // load icon ID
    bra end_string
not_mastered:
    lda #$FF                    // add space
end_string:
    sta.w $2180                 // add it to string
    stz.w $2180                 // end string
    rtl
}
